__submodules__ = ["installer"]

# <AUTOGEN_INIT>
from kslurm.installer.installer import (
    METADATA_URL,
    install,
    run_installation,
)

__all__ = ["METADATA_URL", "install", "run_installation"]

# </AUTOGEN_INIT>
