from test.args.models.basic import basic_attr, basic_dict
from test.args.models.no_defaults import no_default_dict, no_default_attr
from test.args.models.positional_and_shape import pos_and_shape

models = [basic_attr, basic_dict, no_default_attr, no_default_dict, pos_and_shape]
