from __future__ import absolute_import

from rich.theme import Theme

default = Theme(
    {
        "heading": "green bold",
        "hot": "deep_pink3",
        "grey": "grey54",
        "syntax_col": "cyan bold",
        "examples_col": "grey54",
        "default_col": "deep_pink3",
        "values_col": "italic",
    }
)
